@@ -3,30 +3,25 @@
 # Assume you have a bmp module with BMP180 class with read() method.
 from sensors.bmp180 import BMP180 as sensor
 
-from pyhap.accessory import Accessory, Category
-import pyhap.loader as loader
+from pyhap.accessory import Accessory
+from pyhap.const import CATEGORY_SENSOR
 
 
 class BMP180(Accessory):
 
-    category = Category.SENSOR
+    category = CATEGORY_SENSOR
 
     def __init__(self, *args, **kwargs):
-        super(BMP180, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
-        self.temp_char = self.get_service("TemperatureSensor")\
-                             .get_characteristic("CurrentTemperature")
+        serv_temp = self.add_preload_service('TemperatureSensor')
+        self.char_temp = serv_temp.get_characteristic('CurrentTemperature')
 
         self.sensor = sensor()
 
-    def _set_services(self):
-        super(BMP180, self)._set_services()
-        self.add_service(
-            loader.get_serv_loader().get("TemperatureSensor"))
-
     def __getstate__(self):
-        state = super(BMP180, self).__getstate__()
-        state["sensor"] = None
+        state = super().__getstate__()
+        state['sensor'] = None
         return state
 
     def __setstate__(self, state):
@@ -36,4 +31,4 @@ class BMP180(Accessory):
     def run(self):
         while not self.run_sentinel.wait(30):
             temp, _pressure = self.sensor.read()
-            self.temp_char.set_value(temp)
+            self.char_temp.set_value(temp)
