@@ -1,8 +1,8 @@
 # An Accessory for viewing/controlling the status of a Mac display.
 import subprocess
 
-from pyhap.accessory import Accessory, Category
-import pyhap.loader as loader
+from pyhap.accessory import Accessory
+from pyhap.const import CATEGORY_SWITCH
 
 
 def get_display_state():
@@ -23,19 +23,14 @@ class DisplaySwitch(Accessory):
     of the Mac that this code is running on.
     """
 
-    category = Category.SWITCH
+    category = CATEGORY_SWITCH
 
     def __init__(self, *args, **kwargs):
-        super(DisplaySwitch, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
-        self.display = self.get_service("Switch")\
-                           .get_characteristic("On")
-        self.display.setter_callback = self.set_display
-
-    def _set_services(self):
-        super(DisplaySwitch, self)._set_services()
-        self.add_service(
-            loader.get_serv_loader().get("Switch"))
+        serv_switch = self.add_preload_service('Switch')
+        self.display = serv_switch.configure_char(
+            'On', setter_callback=self.set_display)
 
     def run(self):
         while not self.run_sentinel.wait(1):
