@@ -3,28 +3,23 @@
 
 import tsl2591
 
-from pyhap.accessory import Accessory, Category
-import pyhap.loader as loader
+from pyhap.accessory import Accessory
+from pyhap.const import CATEGORY_SENSOR
 
 
 class TSL2591(Accessory):
-    category = Category.SENSOR
+    category = CATEGORY_SENSOR
 
     def __init__(self, *args, **kwargs):
-        super(TSL2591, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
-        self.lux_char = self.get_service("LightSensor") \
-            .get_characteristic("CurrentAmbientLightLevel")
+        serv_light = self.add_preload_service('LightSensor')
+        self.char_lux = serv_light.configure_char('CurrentAmbientLightLevel')
 
         self.tsl = tsl2591.Tsl2591()
 
-    def _set_services(self):
-        super(TSL2591, self)._set_services()
-        self.add_service(
-            loader.get_serv_loader().get("LightSensor"))
-
     def __getstate__(self):
-        state = super(TSL2591, self).__getstate__()
+        state = super().__getstate__()
         state["tsl"] = None
         return state
 
@@ -36,5 +31,5 @@ class TSL2591(Accessory):
         while not self.run_sentinel.wait(10):
             full, ir = self.tsl.get_full_luminosity()
             lux = min(max(0.001, self.tsl.calculate_lux(full, ir)), 10000)
-            self.lux_char.set_value(lux)
+            self.char_lux.set_value(lux)
 
