@@ -1,106 +1,26 @@
-import threading
-import logging
+import asyncio
 import itertools
+import logging
 import struct
-from os import urandom
+import threading
 
 import ed25519
-import base36
-from pyqrcode import QRCode
 
-import pyhap.util as util
-from pyhap.loader import get_serv_loader
+from pyhap import util, SUPPORT_QR_CODE
+from pyhap.const import (
+    STANDALONE_AID, HAP_REPR_AID, HAP_REPR_IID, HAP_REPR_SERVICES,
+    HAP_REPR_VALUE, CATEGORY_OTHER, CATEGORY_BRIDGE)
+from pyhap.iid_manager import IIDManager
+from pyhap.loader import get_loader
+
+if SUPPORT_QR_CODE:
+    import base36
+    from pyqrcode import QRCode
 
 logger = logging.getLogger(__name__)
 
 
-class Category:
-    """Known category values.
-
-    Category is a hint to iOS clients about what "type" of Accessory this represents,
-    for UI only.
-    """
-    OTHER = 1
-    BRIDGE = 2
-    FAN = 3
-    GARAGE_DOOR_OPENER = 4
-    LIGHTBULB = 5
-    DOOR_LOCK = 6
-    OUTLET = 7
-    SWITCH = 8
-    THERMOSTAT = 9
-    SENSOR = 10
-    ALARM_SYSTEM = 11
-    DOOR = 12
-    WINDOW = 13
-    WINDOW_COVERING = 14
-    PROGRAMMABLE_SWITCH = 15
-    RANGE_EXTENDER = 16
-    CAMERA = 17
-
-
-# Standalone accessory ID (i.e. not bridged)
-STANDALONE_AID = 1
-
-
-class IIDManager(object):
-    """Maintains a mapping between Service/Characteristic objects and IIDs."""
-
-    def __init__(self):
-        """Initialise an empty instance."""
-        self.iids = {}
-        self.reverse_iids = {}
-
-    def assign(self, obj):
-        """Assign an IID to the given object.
-
-        If the object already has an assigned ID, log a warning and do nothing.
-
-        :param obj: The object that will be assigned an IID.
-        :type obj: Service or Characteristic
-        """
-        if obj in self.reverse_iids:
-            logger.warning("The given Service or Characteristic with UUID %s "
-                           "already has an assigned IID %s, ignoring.",
-                           obj.type_id, self.reverse_iids[obj])
-            return
-        iid = len(self.iids) + 1
-        self.iids[iid] = obj
-        self.reverse_iids[obj] = iid
-
-    def remove(self, obj=None, iid=None):
-        """Remove an object or an object with the given IID."""
-        if obj is not None:
-            iid = self.reverse_iids.pop(obj, None)
-            if iid is None:
-                logger.error("Object %s not found.", obj)
-                return
-            del self.iids[iid]
-        else:
-            obj = self.iids.pop(iid, None)
-            if obj is None:
-                logger.error("IID %s not found.", iid)
-                return
-            del self.reverse_iids[obj]
-
-    def get_iid(self, obj):
-        """Get the IID assigned to the given object.
-
-        :return: IID assigned to the given object or None if the object is not found.
-        :rtype: int
-        """
-        return self.reverse_iids.get(obj)
-
-    def get_obj(self, iid):
-        """Get the object that is assigned the given IID.
-
-        :return: The object with the given IID or None if no object has that IID.
-        :rtype: Service or Characteristic
-        """
-        return self.iids.get(iid)
-
-
-class Accessory(object):
+class Accessory:
     """A representation of a HAP accessory.
 
     Inherit from this class to build your own accessories.
@@ -109,7 +29,7 @@ class Accessory(object):
     Use this to set your HAP services.
     """
 
-    category = Category.OTHER
+    category = CATEGORY_OTHER
 
     @classmethod
     def create(cls, display_name, pincode, aid=STANDALONE_AID):
@@ -152,9 +72,11 @@ class Accessory(object):
         self.reachable = True
         self._pincode = pincode
         self._setup_id = setup_id
-        self.broker = None
+        self.driver = None
         # threading.Event that gets set when the Accessory should stop.
         self.run_sentinel = None
+        self.event_loop = None
+        self.aio_stop_event = None
 
         sk, vk = ed25519.create_keypair()
         self.private_key = sk
@@ -163,6 +85,8 @@ class Accessory(object):
         self.services = []
         self.iid_manager = iid_manager or IIDManager()
 
+        self.add_info_service()
+
         self._set_services()
 
     def __repr__(self):
@@ -173,45 +97,73 @@ class Accessory(object):
 
     def __getstate__(self):
         state = self.__dict__.copy()
-        state["broker"] = None
-        state["run_sentinel"] = None
+        state['driver'] = None
+        state['run_sentinel'] = None
         return state
 
     @property
     def setup_id(self):
-        if not getattr(self, '_setup_id', None):
+        if self._setup_id is None:
             self._setup_id = util.generate_setup_id()
         return self._setup_id
 
     @property
     def pincode(self):
-        if not getattr(self, '_pincode', None):
+        if self._pincode is None:
             self._pincode = util.generate_pincode()
         return self._pincode
 
     def _set_services(self):
         """Sets the services for this accessory.
 
-        The default implementation adds only the AccessoryInformation services
-        and sets its Name characteristic to the Accessory's display name.
-
-        .. note:: When inheriting from Accessory and overriding this method,
-            always call the base implementation first, as it reserves IID of
-            1 for the Accessory Information service (HAP requirement).
+        .. deprecated:: 2.0
+           Initialize the service inside the accessory `init` method instead.
         """
-        info_service = get_serv_loader().get("AccessoryInformation")
-        info_service.get_characteristic("Name")\
-                    .set_value(self.display_name, False)
-        info_service.get_characteristic("Manufacturer")\
-                    .set_value("Default-Manufacturer", False)
-        info_service.get_characteristic("Model")\
-                    .set_value("Default-Model", False)
-        info_service.get_characteristic("SerialNumber")\
-                    .set_value("Default-SerialNumber", False)
-        # FIXME: Need to ensure AccessoryInformation is with IID 1.
-        self.add_service(info_service)
+        pass
 
-    def set_sentinel(self, run_sentinel):
+    def add_info_service(self):
+        """Helper method to add the required `AccessoryInformation` service.
+
+        Called in `__init__` to be sure that it is the first service added.
+        May be overridden.
+        """
+        serv_info = get_loader().get_service('AccessoryInformation')
+        serv_info.configure_char('Name', value=self.display_name)
+        serv_info.configure_char('SerialNumber', value='default')
+        self.add_service(serv_info)
+
+    def set_info_service(self, firmware_revision=None, manufacturer=None,
+                         model=None, serial_number=None):
+        """Quick assign basic accessory information."""
+        serv_info = self.get_service('AccessoryInformation')
+        if firmware_revision:
+            serv_info.configure_char(
+                'FirmwareRevision', value=firmware_revision)
+        if manufacturer:
+            serv_info.configure_char('Manufacturer', value=manufacturer)
+        if model:
+            serv_info.configure_char('Model', value=model)
+        if serial_number:
+            if len(serial_number) >= 1:
+                serv_info.configure_char('SerialNumber', value=serial_number)
+            else:
+                logger.warning(
+                    "Couldn't add SerialNumber for %s. The SerialNumber must "
+                    "be at least one character long.", self.display_name)
+
+    def add_preload_service(self, service, chars=None):
+        """Create a service with the given name and add it to this acc."""
+        loader = get_loader()
+        service = loader.get_service(service)
+        if chars:
+            chars = chars if isinstance(chars, list) else [chars]
+            for char_name in chars:
+                char = loader.get_char(char_name)
+                service.add_characteristic(char)
+        self.add_service(service)
+        return service
+
+    def set_sentinel(self, run_sentinel, aio_stop_event, event_loop):
         """Assign a run sentinel that can signal stopping.
 
         The run sentinel is a threading.Event object that can be used to manage
@@ -224,6 +176,8 @@ class Accessory(object):
         ...    sensor.readTemperature()
         """
         self.run_sentinel = run_sentinel
+        self.aio_stop_event = aio_stop_event
+        self.event_loop = event_loop
 
     def config_changed(self):
         """Notify the accessory about configuration changes.
@@ -231,7 +185,7 @@ class Accessory(object):
         These include new services or updated characteristic values, e.g.
         the Name of a service changed.
 
-        This method also notifies the broker about the change, so that it can
+        This method also notifies the driver about the change, so that it can
         publish the changes to the world.
 
         .. note:: If you are changing the configuration of a bridged accessory
@@ -240,7 +194,7 @@ class Accessory(object):
 
         """
         self.config_version += 1
-        self.broker.config_changed()
+        self.driver.config_changed()
 
     def add_service(self, *servs):
         """Add the given services to this Accessory.
@@ -256,6 +210,7 @@ class Accessory(object):
         for s in servs:
             self.services.append(s)
             self.iid_manager.assign(s)
+            s.broker = self
             for c in s.characteristics:
                 self.iid_manager.assign(c)
                 c.broker = self
@@ -275,8 +230,8 @@ class Accessory(object):
         """
         return next((s for s in self.services if s.display_name == name), None)
 
-    def set_broker(self, broker):
-        self.broker = broker
+    def set_driver(self, driver):
+        self.driver = driver
 
     def add_paired_client(self, client_uuid, client_public):
         """Adds the given client to the set of paired clients.
@@ -301,7 +256,6 @@ class Accessory(object):
     def paired(self):
         return len(self.paired_clients) > 0
 
-    @property
     def xhm_uri(self):
         """Generates the X-HM:// uri (Setup Code URI)
 
@@ -313,7 +267,7 @@ class Accessory(object):
         value_low |= 1 << 28
         struct.pack_into('>L', buffer, 4, value_low)
 
-        if self.category == Category.OTHER:
+        if self.category == CATEGORY_OTHER:
             buffer[4] = buffer[4] | 1 << 7
 
         value_high = self.category >> 1
@@ -325,19 +279,6 @@ class Accessory(object):
 
         return 'X-HM://' + encoded_payload + self.setup_id
 
-    @property
-    def qr_code(self):
-        """Generate a QR code for paring with this accessory.
-
-        :rtype: QRCode
-        """
-        return QRCode(self.xhm_uri)
-
-    def print_qr(self):
-        """Print the setup code in QR format to console.
-        """
-        print(self.qr_code.terminal(), flush=True)
-
     def get_characteristic(self, aid, iid):
         """Get the characteristic for the given IID.
 
@@ -348,7 +289,7 @@ class Accessory(object):
 
         return self.iid_manager.get_obj(iid)
 
-    def to_HAP(self, iid_manager=None):
+    def to_HAP(self):
         """A HAP representation of this Accessory.
 
         :return: A HAP representation of this accessory. For example:
@@ -365,16 +306,52 @@ class Accessory(object):
 
         :rtype: dict
         """
-        iid_manager = iid_manager or self.iid_manager
-        services_HAP = [s.to_HAP(iid_manager) for s in self.services]
-        hap_rep = {"aid": self.aid, "services": services_HAP, }
-        return hap_rep
+        return {
+            HAP_REPR_AID: self.aid,
+            HAP_REPR_SERVICES: [s.to_HAP() for s in self.services],
+        }
 
     def setup_message(self):
-        print('Setup payload: %s' % self.xhm_uri, flush=True)
-        print('Scan this code with your HomeKit app on your iOS device:', flush=True)
-        self.print_qr()
-        print('Or enter this code in your HomeKit app on your iOS device: %s' % self.pincode.decode())
+        """Print setup message to console.
+
+        For QRCode `base36`, `pyqrcode` are required.
+        Installation through `pip install HAP-python[QRCode]`
+        """
+        if SUPPORT_QR_CODE:
+            xhm_uri = self.xhm_uri()
+            print('Setup payload: {}'.format(xhm_uri), flush=True)
+            print('Scan this code with your HomeKit app on your iOS device:',
+                  flush=True)
+            print(QRCode(xhm_uri).terminal(quiet_zone=2), flush=True)
+            print('Or enter this code in your HomeKit app on your iOS device: '
+                  '{}'.format(self.pincode.decode()))
+        else:
+            print('To use the QR Code feature, use \'pip install '
+                  'HAP-python[QRCode]\'')
+            print('Enter this code in your HomeKit app on your iOS device: {}'
+                  .format(self.pincode.decode()))
+
+    def run_at_interval(seconds):
+        """Decorator that runs decorated method in a while loop, which repeats every
+        ``seconds`` until the ``Accessory.run_sentinel`` is set.
+
+        .. code-block:: python
+
+            @Accessory.run_at_interval(3)
+            def run(self):
+                print("Hello again world!")
+
+        :param seconds: The amount of seconds to wait for the event to be set.
+            Determines the interval on which the decorated method will be called.
+        :type seconds: float
+        """
+        # decorator returns a decorator with the argument it got
+        def _repeat(func):
+            def _wrapper(self, *args, **kwargs):
+                while not self.run_sentinel.wait(seconds):
+                    func(self, *args, **kwargs)
+            return _wrapper
+        return _repeat
 
     def run(self):
         """Called when the Accessory should start doing its thing.
@@ -388,14 +365,14 @@ class Accessory(object):
         """
         pass
 
-    # Broker
+    # Driver
 
     def publish(self, value, sender):
-        """Append AID and IID of the sender and forward it to the broker.
+        """Append AID and IID of the sender and forward it to the driver.
 
         Characteristics call this method to send updates.
 
-        .. note:: The method will not fail if the broker is not set - it will do nothing.
+        .. note:: The method will not fail if the driver is not set - it will do nothing.
 
         :param data: Data to publish, usually from a Characteristic.
         :type data: dict
@@ -403,24 +380,56 @@ class Accessory(object):
         :param sender: The Service or Characteristic from which the call originated.
         :type: Service or Characteristic
         """
-        if self.broker is None:
+        if self.driver is None:
             return
 
         acc_data = {
-            "aid": self.aid,
-            "iid": self.iid_manager.get_iid(sender),
-            "value": value,
+            HAP_REPR_AID: self.aid,
+            HAP_REPR_IID: self.iid_manager.get_iid(sender),
+            HAP_REPR_VALUE: value,
         }
-        self.broker.publish(acc_data)
+        self.driver.publish(acc_data)
 
 
-class Bridge(Accessory):
+class AsyncAccessory(Accessory):
+
+    def run_at_interval(seconds):
+        """Decorator that runs decorated method in a while loop, which repeats every
+        ``seconds`` until the ``Accessory.aio_stop_event`` is set.
+
+        .. code-block:: python
+
+            @AsyncAccessory.run_at_interval(3)
+            async def run(self):
+                print("Hello again world!")
+
+        :param seconds: The amount of seconds to wait for the event to be set.
+            Determines the interval on which the decorated method will be called.
+        :type seconds: float
+        """
+        # decorator returns a decorator with the argument it got
+        def _repeat(func):
+            async def _wrapper(self, *args, **kwargs):
+                while not await util.event_wait(self.aio_stop_event,
+                                                seconds,
+                                                self.event_loop):
+                    await func(self, *args, **kwargs)
+            return _wrapper
+        return _repeat
+
+    async def run(self):
+        """Override in the implementation if needed.
+        """
+        pass
+
+
+class Bridge(AsyncAccessory):
     """A representation of a HAP bridge.
 
     A `Bridge` can have multiple `Accessories`.
     """
 
-    category = Category.BRIDGE
+    category = CATEGORY_BRIDGE
 
     def __init__(self, display_name, mac=None, pincode=None,
                  iid_manager=None, setup_id=None):
@@ -428,16 +437,15 @@ class Bridge(Accessory):
         # A Bridge cannot be Bridge, hence talks directly to HAP clients.
         # Thus, we need a mac.
         mac = mac or util.generate_mac()
-        super(Bridge, self).__init__(display_name, aid=aid, mac=mac,
-                                     pincode=pincode, iid_manager=iid_manager,
-                                     setup_id=setup_id)
+        super().__init__(display_name, aid=aid, mac=mac, pincode=pincode,
+                         iid_manager=iid_manager, setup_id=setup_id)
         self.accessories = {}  # aid: acc
 
-    def set_sentinel(self, run_sentinel):
+    def set_sentinel(self, run_sentinel, aio_stop_event, event_loop):
         """Set the same sentinel to all contained accessories."""
-        super(Bridge, self).set_sentinel(run_sentinel)
+        super().set_sentinel(run_sentinel, aio_stop_event, event_loop)
         for acc in self.accessories.values():
-            acc.set_sentinel(run_sentinel)
+            acc.set_sentinel(run_sentinel, aio_stop_event, event_loop)
 
     def add_accessory(self, acc):
         """Add the given ``Accessory`` to this ``Bridge``.
@@ -453,11 +461,11 @@ class Bridge(Accessory):
         :param acc: The ``Accessory`` to be bridged.
         :type acc: Accessory
 
-        :raise ValueError: When the given ``Accessory`` is of category ``Category.BRIDGE``
+        :raise ValueError: When the given ``Accessory`` is of category ``CATEGORY_BRIDGE``
             or if the AID of the ``Accessory`` clashes with another ``Accessory`` already in this
             ``Bridge``.
         """
-        if acc.category == Category.BRIDGE:
+        if acc.category == CATEGORY_BRIDGE:
             raise ValueError("Bridges cannot be bridged")
 
         if acc.aid is None:
@@ -469,20 +477,20 @@ class Bridge(Accessory):
 
         self.accessories[acc.aid] = acc
 
-    def set_broker(self, broker):
-        super(Bridge, self).set_broker(broker)
+    def set_driver(self, driver):
+        super().set_driver(driver)
         for _, acc in self.accessories.items():
-            acc.broker = broker
+            acc.driver = driver
 
-    def to_HAP(self, iid_manager=None):
+    def to_HAP(self):
         """Returns a HAP representation of itself and all contained accessories.
 
         .. seealso:: Accessory.to_HAP
         """
-        hap_rep = [super(Bridge, self).to_HAP(iid_manager), ]
+        hap_rep = [super().to_HAP()]
 
-        for _, acc in self.accessories.items():
-            hap_rep.append(acc.to_HAP(iid_manager))
+        for acc in self.accessories.values():
+            hap_rep.append(acc.to_HAP())
 
         return hap_rep
 
@@ -498,19 +506,31 @@ class Bridge(Accessory):
 
         return acc.get_characteristic(aid, iid)
 
-    def run(self):
-        """Creates and starts a new thread for each of the contained accessories' run
-            method.
+    async def _wrap_in_thread(self, method):
+        """Coroutine which starts the given method in a thread.
         """
+        # Not going through event_loop.run_in_executor, because this thread may never
+        # terminate.
+        threading.Thread(target=method).start()
+
+    async def run(self):
+        """Schedule tasks for each of the accessories' run method.
+        """
+        tasks = []
         for acc in self.accessories.values():
-            threading.Thread(target=acc.run).start()
+            if isinstance(acc, AsyncAccessory):
+                task = self.event_loop.create_task(acc.run())
+            else:
+                task = self.event_loop.create_task(self._wrap_in_thread(acc.run))
+            tasks.append(task)
+        await asyncio.gather(*tasks, loop=self.event_loop)
 
     def stop(self):
         """Calls stop() on all contained accessories."""
-        super(Bridge, self).stop()
+        super().stop()
         for acc in self.accessories.values():
             acc.stop()
 
 
 def get_topic(aid, iid):
-    return str(aid) + "." + str(iid)
+    return str(aid) + '.' + str(iid)
