@@ -1,3 +1,4 @@
+import asyncio
 import socket
 import random
 import binascii
@@ -113,3 +114,21 @@ tohex = bytes.hex if sys.version_info >= (3, 5) else b2hex
 fromhex = bytes.fromhex if sys.version_info >= (3, 5) else hex2b
 """Python-version-agnostic fromhex function. Equivalent to bytes.fromhex in python 3.5+.
 """
+
+async def event_wait(event, timeout, loop=None):
+    """Wait for the given event to be set or for the timeout to expire.
+
+    :param event: The event to wait for.
+    :type event: asyncio.Event
+
+    :param timeout: The timeout for which to wait, in seconds.
+    :type timeout: float
+
+    :return: ``event.is_set()``
+    :rtype: bool
+    """
+    try:
+        await asyncio.wait_for(event.wait(), timeout, loop=loop)
+    except asyncio.TimeoutError:
+        pass
+    return event.is_set()
\ No newline at end of file
